###################################################################################
# Phabricator QA tool
# Copyright (C) 2015  Luis Araujo <luis.araujo@collabora.co.uk>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  US
###################################################################################

import os
from argparse import ArgumentParser
from requests.exceptions import ConnectionError

from pqa_tool.version import __version__
from pqa_tool.commands.task import TaskCmd
from pqa_tool.commands.update import UpdateCmd
from pqa_tool.commands.query import QueryCmd
from pqa_tool.commands.paste import PasteCmd
from pqa_tool.commands.showpaste import ShowPasteCmd
from pqa_tool.commands.createtask import CreateTaskCmd
from pqa_tool.commands.whoami import WhoAmICmd
from pqa_tool.commands.createproj import CreateProjCmd

class Cli(object):
    """Command line interface using argparse"""

    def __init__(self):
        self.parser = ArgumentParser(description="pqa v{}".format(__version__))
        self.parser.add_argument('-c', '--config', metavar='ARCCONFIG',
                                 default=os.getenv('HOME') + '/.arcrc',
                                 help="Set the arcrc configuration file to use")
        # Sub-commands
        subparsers = self.parser.add_subparsers()

        # Task
        task_parser = subparsers.add_parser('task', help="Task operations")
        task_parser.add_argument('task_id', type=int, metavar='TASK_ID',
                                 help="ID of the task")
        task_parser.add_argument('-c', '--show-comments', action='store_true',
                                 help="Show task comments")
        task_parser.set_defaults(func=task)

        # Update Task
        update_parser = subparsers.add_parser('update', help="Update task")
        update_parser.add_argument('task_id', type=int, metavar='TASK_ID',
                                   help="ID of the task")
        update_parser.add_argument('-t', '--title', type=str, metavar='TITLE',
                                   help="Update title")
        update_parser.add_argument('-d', '--description', type=str,
                                   metavar='DESCRIPTION',
                                   help="Update description")
        update_parser.add_argument('-o', '--owner', type=str,
                                   metavar='OWNER', help="Update owner")
        update_parser.add_argument('-m', '--member', action='append', type=str,
                                   metavar='MEMBER', help="Add member (can "
                                   "be given multiple times)")
        update_parser.add_argument('-r', '--priority',
                                   choices=['low', 'normal', 'triage', 
                                            'high', 'blocker'],
                                   help="Update priority")
        update_parser.add_argument('-p', '--project', action='append', type=str,
                                   metavar='PROJECT', help="Set project (can be "
                                   "given multiple times)")
        update_parser.add_argument('-s', '--status', type=str,
                                   help="Update task status")
        update_parser.add_argument('-c', '--comment', type=str, metavar='COMMENT',
                                   help="Add comment")
        update_parser.add_argument('-f', '--field', type=str,
                                   action="append", metavar='FIELD',
                                   help="Set auxiliary/extra fields in the "
                                   "form of 'field=value' (can be given "
                                   "multiple times)")
        update_parser.set_defaults(func=update)

        # Query Task
        query_parser = subparsers.add_parser('query', help="Query tasks")
        query_parser.add_argument('-i', '--id', action='append', type=int,
                                  metavar='ID', help="Task IDs (can be given "
                                  "multiple times)")
        query_parser.add_argument('-o', '--owner', action='append', type=str,
                                  metavar='OWNER', help="Search by owner (can be "
                                  "given multiple times)")
        query_parser.add_argument('-a', '--author', action='append', type=str,
                                  metavar='AUTHOR', help="Author name (can be "
                                  "given multiple times)")
        query_parser.add_argument('-p', '--project', action='append', type=str,
                                  metavar='PROJECT', help="Project name (can be "
                                  "given multiple times)")
        query_parser.add_argument('-m', '--member', action='append', type=str,
                                  metavar='MEMBER', help="Query by member (can "
                                  "be given multiple times)")
        query_parser.add_argument('-f', '--fulltext', type=str, metavar='FULLTEXT',
                                  help='Full text')
        query_parser.add_argument('-s', '--status', type=str, help='Task status')
        query_parser.add_argument('-r', '--order',
                                  choices=['priority', 'created', 'modified'],
                                  help='Query order')
        query_parser.add_argument('-l', '--limit', type=int, metavar='LIMIT',
                                  help="Limit for the query list")
        query_parser.add_argument('-e', '--offset', type=int, metavar='OFFSET',
                                  help="Query offset")
        query_parser.set_defaults(func=query)

        # Create Task
        createtask_parser = subparsers.add_parser('createtask', help="Create task")
        createtask_parser.add_argument('title', type=str, metavar='TITLE',
                                       help="Title for the new task")
        createtask_parser.add_argument('-d', '--description', type=str,
                                       metavar='DESCRIPTION',
                                       help="Description for the new task")
        createtask_parser.add_argument('-o', '--owner', type=str,
                                       metavar='OWNER', help="Set owner")
        createtask_parser.add_argument('-m', '--member', action='append', type=str,
                                       metavar='MEMBER', help="Member name (can "
                                       "be given multiple times)")
        createtask_parser.add_argument('-r', '--priority',
                                       choices=['low', 'normal', 'triage',
                                                'high', 'blocker'],
                                       help="Set priority")
        createtask_parser.add_argument('-p', '--project', action='append',
                                       type=str, metavar='PROJECT', help="Project"
                                       " name (can be given multiple times)")
        createtask_parser.add_argument('-f', '--field', type=str,
                                       action="append", metavar='FIELD',
                                       help="Set auxiliary/extra fields in the "
                                       "form of 'field=value' (can be given "
                                       "multiple times)")
        createtask_parser.set_defaults(func=createtask)

        # Create Project
        createproj_parser = subparsers.add_parser('createproj',
                                                  help="Create project")
        createproj_parser.add_argument('name', type=str, metavar='NAME',
                                       help="Name for the new project")
        createproj_parser.add_argument('-m', '--member', type=str,
                                       action="append", metavar='MEMBER',
                                       help="Add member to the new project (can "
                                       "be given multiple times)")
        createproj_parser.add_argument('-i', '--icon', type=str, metavar='ICON',
                                       help='Icon for the project')
        createproj_parser.add_argument('-c', '--color', type=str, metavar='COLOR',
                                       help='Color to be used for the project')
        createproj_parser.add_argument('-t', '--tag', type=str,
                                       action="append", metavar='TAG',
                                       help="Add hashtag to the new project (can "
                                       "be given multiple times)")
        createproj_parser.set_defaults(func=createproj)

        # Paste
        paste_parser = subparsers.add_parser('paste', help="Paste content")
        paste_parser.add_argument('-c', '--content', type=str, metavar='CONTENT',
                                  help="Content to paste (reads from stdin if no "
                                  "content passed)")
        paste_parser.add_argument('-t', '--title', type=str, metavar='TITLE',
                                  help="Add title")
        paste_parser.add_argument('-l', '--language', type=str, metavar='LANGUAGE',
                                  help="Specify the language")
        paste_parser.set_defaults(func=paste)

        # Show paste
        showpaste_parser = subparsers.add_parser('showpaste', help="Show paste")
        showpaste_parser.add_argument('paste_id', type=int, metavar='PASTE_ID',
                                      help="ID of the paste to show")
        showpaste_parser.add_argument('-s', '--show-info', action='store_true',
                                      help="Show paste info")
        showpaste_parser.set_defaults(func=showpaste)

        # Who am I?
        whoami_parser = subparsers.add_parser('whoami', help="Show authenticated "
                                              "user name")
        whoami_parser.set_defaults(func=whoami)

    def run(self):
        args = self.parser.parse_args()
        try:
            args.func(args)
        except ConnectionError as e:
            print e
            exit(2)
        except KeyboardInterrupt:
            pass

def task(args):
    TaskCmd(args).run()

def createtask(args):
    CreateTaskCmd(args).run()

def update(args):
    UpdateCmd(args).run()

def query(args):
    QueryCmd(args).run()

def paste(args):
    PasteCmd(args).run()

def showpaste(args):
    ShowPasteCmd(args).run()

def whoami(args):
    WhoAmICmd(args).run()

def createproj(args):
    CreateProjCmd(args).run()
