###################################################################################
# Phabricator Conduit Python API.
# Copyright (C) 2016  Luis Araujo <luis.araujo@collabora.co.uk>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  US
###################################################################################

import json
import os.path
import requests

class Conduit(object):
    """Conduit class"""

    def __init__(self, api_url, api_token):
        self._api_token = api_token
        self._api_url = api_url
        # Persist the API token in a request Session.
        self._session = requests.Session()
        self._session.params.update({ 'api.token': self._api_token })

    def call(self, method, **params):
        """This method builds the correct parameter format and call the 
        conduit API method with it."""

        p = {}
        for param in params:
            if type(params[param]) == list:
                p.update(_build_params_list(param, params[param]))
            elif type(params[param]) == dict:
                p.update(_build_params_hash(param, params[param]))
            else:
                p.update({ param: params[param] })

        res = self._session.get(os.path.join(self._api_url, method), params=p)
        return json.loads(res.content)

    @classmethod
    def from_config(cls, config_file=os.getenv('HOME') + '/.arcrc'):
        """This class method is used to fetch the required conduit API
        information from a configuration file (default to $HOME/.arcrc if not
        explicitly passed."""

        try:
            with open(config_file) as conf_data:
                arcrc_config = json.loads(conf_data.read())
        except IOError as err:
            raise ConfigException(err)
        except EnvironmentError as err:
            raise ConfigException(err)

        config = arcrc_config.get('config', None)
        if not config:
            raise ConfigException(
                "No 'config' field found in {}".format(config_file))
        
        default_host = config.get('default', None)
        if not default_host:
            raise ConfigException(
                "No 'default' host found in {}".format(config_file))

        if 'hosts' not in arcrc_config:
            raise ConfigException(
                "No 'hosts' field found in {}".format(config_file))

        api_url = api_token = None
        for host in arcrc_config.get('hosts', {}).keys():
            if host.startswith(default_host):
                api_url = host
                api_token = arcrc_config['hosts'][host].get('token', None)

        # Check that api url and token fields were found. 
        if not api_url:
            raise ConfigException('API URL not found in {}'.format(config_file))
        if not api_token:
            raise ConfigException('API Token not found in {}'.format(config_file))

        return cls(api_url, api_token)

class ConfigException(Exception):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.__class__.__name__

def _build_params_list(param, lst):
    """Build a parameter list that can be understood by the python request call.

    A list parameter of the form:
    - ids=[1,2,3]
    Translates to a hash like =>
    - { 'ids[0]'=1, 'ids[1]'=2, 'ids[2]'=3 }.
    """
    kv = {}
    for i, arg in enumerate(lst):
        kv.update({ '{}[{}]'.format(param, i): arg })
    return kv

def _build_params_hash(param, hsh):
    """Build a parameter hash that can be understood by the python request call.

    A hash parameter of the form:
    - auxiliary={'image:': 'singularity', 'package': 'pqa' }
    Translates to a hash like =>
    - { 'auxiliary[image]'='singularity', 'auxiliary[package]'='pqa' }.
    """
    kv = {}
    for key in hsh:
        kv.update({ '{}[{}]'.format(param, key): hsh[key] })
    return kv
